//Version: 1.0
using System;
using System.Collections.Generic;
using System.Text;
using HandHeldProducts.Embedded.Decoding;
using HandHeldProducts.Embedded.Hardware;

namespace Dolphin
{
    public class Dolphin:IDisposable 
    {
        // 1.0  first version
        public static double version = 1.0;

        // private variables
        DecodeAssembly decoder;
        private object[] DecodeEventObject;
        private bool scanning;

        // results
        private int result;
        private string aimid;
        private string aimmodifier;
        private string codeid;
        private int length;
        private string message;
        private byte[] rawdata;


        // public events
        public event EventHandler Decode; //The event handler name must match the event name.

        //constructor
        public Dolphin()
        {
            decoder = new DecodeAssembly();
            decoder.DecodeEvent += DecodeEvent;
            DecodeEventObject = new object[] { this, "Decode" };
            scanning = false;
        }


        // event handlers
        private void DecodeEvent(object sender, DecodeAssembly.DecodeEventArgs e)
        {
            scanning = false;
            aimid = e.AimId;
            aimmodifier = e.AimModifier;
            codeid = e.CodeId;
            length = e.Length;
            message = e.Message;
            rawdata = e.RawData;
            result = (int) e.ResultCode;
            if (Decode != null)
            {
                Decode(DecodeEventObject, null);
            }
        }

        //methods

        public void StartScan()
        {
            scanning = true;
            decoder.ScanBarcode();
        }

        public void StopScan()
        {
            decoder.CancelScanBarcode();
        }


        //properties

        public double DLLVersion
        { get { return version; } }

        public Object ControlRef
        { get { return decoder; } }

        public Object IsScanning
        { get { return scanning; } }


        // scan result properties

        public string AimId
        { get { return aimid; } }

        public string AimModifier
        { get { return aimmodifier; } }

        public string CodeId
        { get { return codeid; } }

        public int Length
        { get { return length; } }

        public string Message
        { get { return message; } }

        public byte[] RawData
        { get { return rawdata; } }

        // Success = 0, BadRegion = 1, DriverError = 2, EngineBusy = 3, MemoryError = 4,
        // NoDecode = 5, NoImage = 6, NoResponse = 7, NotConnected = 8,  ParameterError = 9,
        // UnSupported = 10, NoTrigger = 11, BadSmartImage = 12,  SmartImageTooLarge = 13,
        // TooMuchInterpolation = 14, WrongResultStruct = 15, ThreadError = 16,
        // Cancel = 17, Initialize = -1,
        public int ResultCode
        { get { return result; } }


        // Dispose
        public void Dispose()
        {
            decoder.DecodeEvent -= DecodeEvent;
            decoder.Dispose();
        }

    }
}
